import * as Joi from 'joi';

export enum Article_type {
    MP = 'MP',
    PF = 'PF',
    Piece = 'Piece',
    SE = 'SE'
}

export interface Article {
    reference: string;
    designation: string;
    type_fabrication_achat: string;
    unite_achat_stock: string;
    delai_en_semaine: number;
    prix_standard?: number;
    lot_de_reapprovisionnement?: number;
    stock_mini?: number;
    stock_maxi?: number;
    pourcentage_de_perte?: number;
    inventaire?: number;
    PF_ou_MP_ou_Piece_ou_SE: Article_type
}

export const Article_ = Joi.object({
    reference: Joi.string().alphanum().required().min(4).max(10),
    designation: Joi.string().pattern(new RegExp('^[a-zA-Z0-9 ]*$')).required().max(30),
    type_fabrication_achat: Joi.string().pattern(new RegExp('^[a-zA-Z0-9. ]*$')).required().max(30),
    unite_achat_stock: Joi.string().pattern(new RegExp('^[a-zA-Z0-9 ]*$')).required().max(30),
    delai_en_semaine: Joi.number().integer().required(),
    prix_standard: Joi.number(),
    lot_de_reapprovisionnement: Joi.number().integer(),
    stock_mini: Joi.number().integer(),
    stock_maxi: Joi.number().integer(),
    pourcentage_de_perte: Joi.number(),
    inventaire: Joi.number().integer(),
    PF_ou_MP_ou_Piece_ou_SE: Joi.string().valid(Article_type.MP, Article_type.PF, Article_type.Piece, Article_type.SE)
}); // .id('Article_'); // 'id' is for possible *INTERNAL* reuse, i.e., '#Article_'

Article_.external(async (article: Article) => { // Check uniqueness in database
    console.log("Access to database from primary key: " + article.reference);
    // Code here...
}).validateAsync({
    reference: 'CD100',
    designation: 'camion demenagement bleu',
    type_fabrication_achat: 'fab. par lot',
    unite_achat_stock: 'unite',
    delai_en_semaine: 2,
    lot_de_reapprovisionnement: 200,
    stock_maxi: 600,
    PF_ou_MP_ou_Piece_ou_SE: Article_type.PF
});

interface Lien_de_nomenclature { // 'compose !== composant'
    compose: Article;
    composant: Article;
    quantite_de_composition: number;
}

export const Lien_de_nomenclature_ = Joi.object({
    // 'exist()' <=> 'required()'...
    compose: Article_.exist(),
    // 'compose' et 'composant' ont des valeurs distinctes pour 'reference' :
    composant: Article_.exist().when('compose.reference', {
        // is: Joi.exist(), // Useless, already checked...
        then: Joi.object({reference: Joi.not(Joi.ref('...compose.reference'))})
    }),
    quantite_de_composition: Joi.number().required()
});

const validation = Lien_de_nomenclature_.validate({
    compose: {
        reference: 'CD100',
        designation: 'camion demenagement bleu',
        type_fabrication_achat: 'fab. par lot',
        unite_achat_stock: 'unite',
        delai_en_semaine: 2,
        lot_de_reapprovisionnement: 200,
        stock_maxi: 600,
        PF_ou_MP_ou_Piece_ou_SE: Article_type.PF
    },
    composant: {
        reference: 'P005', // Replace by 'CD100' for test...
        designation: 'phare normal',
        type_fabrication_achat: 'achat par lot',
        unite_achat_stock: 'unite',
        delai_en_semaine: 2,
        prix_standard: 0.75,
        lot_de_reapprovisionnement: 500,
        stock_mini: 750,
        stock_maxi: 1500,
        PF_ou_MP_ou_Piece_ou_SE: Article_type.Piece
    },
    quantite_de_composition: 2
});
if (validation.error === undefined)
    console.log("Succès : " + validation.value.compose.reference + "-" + validation.value.composant.reference);
else
    console.log("Echec : " + validation.error.message);

// const same_object = {
//     reference: 'CD100',
//     designation: 'camion demenagement bleu',
//     type_fabrication_achat: 'fab. par lot',
//     unite_achat_stock: 'unite',
//     delai_en_semaine: 2,
//     lot_de_reapprovisionnement: 200,
//     stock_maxi: 600,
//     PF_ou_MP_ou_Piece_ou_SE: Article_type.PF
// };
// const validation = Lien_de_nomenclature_.validate({
//     compose: same_object,
//     composant: same_object,
//     quantite_de_composition: 2
// });
// if (validation.error === undefined)
//     console.log("Succès : " + validation.value.compose.reference + "-" + validation.value.composant.reference);
// else
//     console.log("Echec : " + validation.error.message);
